// Adaptive control with discrete model
// - first order controled discrete system
// - adaptive control with receding horizon
// Experiments
// - for specific control aim determine the penalization function
// - check performance of the control in ependence on the amount
//   of uncertainty in the system
//
//  Dynamic programming
//  fp = omega + fs  (penalty function, old criterion)
//  f  = E[fp|d(t)] - expectation over y(t+1)
//  fs = min f  =>  u(t+1) - optimal u
// ---------------------------------------------------------------
exec("ScIntro.sce",-1),mode(0)


// VARIABLES TO BE SET
nh=5;                         // length of control interval
nd=30;                        // length of control
y(1)=1; y(2)=1;               // initial condition for output
u(2)=1;                       // initial condition for input
nu=ones(4,2);                 // initial statistics for estimation

k=10;
// y 1 2       u y1  =  criterion
om=[3 4+k     // 1 1
    4 5+k     // 1 2
    4 5+k     // 2 1
    5 6+k];   // 2 2

// y  1  2       u y1  =  system model
th=[.3 .7     // 1 1
    .1 .9     // 1 2
    .6 .4     // 2 1
    .8 .2];   // 2 2

// time loop of adaptive control
for t=1:nd
  // ESTIMATION
  i=2*(u(t+1)-1)+y(t);
  nu(i,y(t+1))=nu(i,y(t+1))+1;
  Et=nu./(sum(nu,2)*ones(1,2));


  fs=zeros(1,2);
  // CONTROL-LAW COMPUTATION
  for i=nh:-1:1
    fp=om+ones(4,1)*fs;         // penalty + reminder from last step
    // expectation
    f=sum((fp.*th),'c');        // expectation over y
    // minimization
    if f(1)<f(3),               // for y(i-1)=1
      us(i,1)=1; fs(1)=f(1);    // optimal control, minimum of criterion
    else
      us(i,1)=2; fs(1)=f(3);    // optimal control, minimum of criterion
    end
    if f(2)<f(4),               // for y(i-1)=2
      us(i,2)=1; fs(2)=f(2);    // optimal control, minimum of criterion
    else
      us(i,2)=2; fs(2)=f(4);    // optimal control, minimum of criterion
    end
  end

  // CONTROL APPLICATION
  u(t+2)=us(1,y(t+1));          // optimal control
  i=2*(u(t+2)-1)+y(t+1);        // row in the model table
  y(t+2)=double(rand(1,1,'unif')>th(i,1))+1; // simulation
end

// RESULTS
s=1:nd;
plot(s,y(s),'ro',s,u(s),'g+')
set(gcf(),'position',[200 40 1000 600])
set(gca(),"data_bounds",[min(s)-1 max(s)+1, .8 2.2])
legend('output','input');
title('Optimal control with discrete model')

